# -*- coding: utf-8 -*-
#
# Tunic
#
# Copyright 2014-2015 TSH Labs <projects@tshlabs.org>
#
# Available under the MIT license. See LICENSE for details.
#

"""
tunic.install
~~~~~~~~~~~~~

Perform installations on remote machines.
"""

import os.path

from .core import FabRunner, ProjectBaseMixin


def _is_iterable(val):
    """Ensure that a value is iterable and not some sort of string"""
    try:
        iter(val)
    except (ValueError, TypeError):
        return False
    else:
        return not isinstance(val, basestring)


class VirtualEnvInstallation(ProjectBaseMixin):
    """Install one or multiple packages into a remote Python virtual environment.

    If the remote virtual environment does not already exist, it will be
    created during the install process.

    The installation can use the standard package index (PyPI) to download
    dependencies, or it can use one or multiple alternative installation
    sources (such as a local PyPI instance, Artifactory, the local file system,
    etc.) and ignore the default index. These two modes are mutually exclusive.

    See :doc:`design` for more information about the expected directory
    structure for deployments.

    .. versionadded:: 0.3.0
    """

    # pylint: disable=too-many-arguments
    def __init__(self, base, packages, sources=None, venv_path=None, runner=None):
        """Set the project base directory, packages to install, and optionally
        alternative sources from which to download dependencies and path to the
        virtualenv tool.

        :param str base: Absolute path to the root of the code deploy on
            the remote server
        :param list packages: A collection of package names to install into
            a remote virtual environment.
        :param list sources: A collection of alternative sources from which
            to install dependencies. These sources should be strings that are
            either URLs or file paths. E.g. 'http://pypi.example.com/simple/'
            or '/tmp/build/mypackages'. Paths and URLs may be mixed in the same
            list of sources.
        :param str venv_path: Optional absolute path to the virtualenv tool on
            the remote server. Required if the virtualenv tool is not in the
            PATH on the remote server.
        :param FabRunner runner: Optional runner to use for executing
            remote and local commands to perform the installation.
        :raises ValueError:  If the base directory isn't specified, if no packages
            are given, packages is not an iterable collection of some kind, or if
            sources is specified but not an iterable collection of some kind.

        .. versionchanged:: 0.4.0
            Allow the path to the ``virtualenv`` script on the remote server to be
            specified.
        """
        super(VirtualEnvInstallation, self).__init__(base)

        if not packages:
            raise ValueError(
                "You must specify at least one package to install")

        if not _is_iterable(packages):
            raise ValueError("Packages must be an iterable")

        if sources is not None and not _is_iterable(sources):
            raise ValueError("Sources must be an iterable")

        self._packages = list(packages)
        self._sources = list(sources) if sources is not None else []
        self._venv_path = venv_path if venv_path is not None else 'virtualenv'
        self._runner = runner if runner is not None else FabRunner()

    def _get_install_sources(self):
        """Construct arguments to use alternative package indexes if
        there were sources supplied, empty string if there were not.
        """
        if not self._sources:
            return ''
        parts = ['--no-index']
        for source in self._sources:
            parts.append("--find-links '{0}'".format(source))
        return ' '.join(parts)

    def install(self, release_id, upgrade=False):
        """Install target packages into a virtual environment.

        If the virtual environment for the given release ID does not
        exist on the remote system, it will be created. The virtual
        environment will be created according to the standard Tunic
        directory structure (see :doc:`design`).

        If ``upgrade=True`` is passed, packages will be updated to the
        most recent version if they are already installed in the virtual
        environment.

        :param str release_id: Timestamp-based identifier for this
            deployment. If this ID corresponds to a virtual environment
            that already exists, packages will be installed into this
            environment.
        :param bool upgrade: Should packages be updated if they are
            already installed in the virtual environment.
        :return: The results of running the installation command using
            Fabric. Note that this return value is a decorated version
            of a string that contains additional meta data about the
            result of the command, in addition to the output generated.
        :rtype: str
        """
        release_path = os.path.join(self._releases, release_id)
        if not self._runner.exists(release_path):
            self._runner.run("{0} '{1}'".format(self._venv_path, release_path))

        cmd = [os.path.join(release_path, 'bin', 'pip'), 'install']
        if upgrade:
            cmd.append('--upgrade')

        sources = self._get_install_sources()
        if sources:
            cmd.append(sources)

        cmd.extend("'{0}'".format(package) for package in self._packages)
        return self._runner.run(' '.join(cmd))


class StaticFileInstallation(ProjectBaseMixin):
    """Install the contents of a local directory into a remote release
    directory.

    If the remote release directory does not already exist, it will be
    created during the install process.

    See :doc:`design` for more information about the expected directory
    structure for deployments.

    .. versionadded:: 0.5.0
    """

    def __init__(self, base, local_path, runner=None):
        """Set the project base directory on the remote server and path to a
        directory of static content to be installed into a remote release
        directory.

        :param str base: Absolute path to the root of the code deploy on
            the remote server
        :param str local_path: Absolute or relative path to a local directory
            whose contents will be copied to a remote release directory.
        :param FabRunner runner: Optional runner to use for executing
            remote and local commands to perform the installation.
        :raises ValueError: If the base directory or local path isn't
            specified.
        """
        super(StaticFileInstallation, self).__init__(base)

        if not local_path:
            raise ValueError("You must specify a local path")

        self._local_path = local_path
        self._runner = runner if runner is not None else FabRunner()

    def install(self, release_id):
        """Install the contents of the local directory into a release directory.

        If the directory for the given release ID does not exist on the remote
        system, it will be created. The directory will be created according to
        the standard Tunic directory structure (see :doc:`design`).

        Note that the name and path of the local directory is irrelevant, only
        the contents of the specified directory will be transferred to the remote
        server. The contents will end up as children of the release directory on
        the remote server.

        :param str release_id: Timestamp-based identifier for this
            deployment. If this ID corresponds to a directory that already
            exists, contents of the local directory will be copied into
            this directory.
        :return: The results of the ``put`` command using Fabric. This return
            value is an iterable of the paths of all files uploaded on the remote
            server.
        """
        release_path = os.path.join(self._releases, release_id)
        if not self._runner.exists(release_path):
            self._runner.run("mkdir -p '{0}'".format(release_path))

        # Make sure to remove any user supplied globs or trailing slashes
        # so that we can ensure exactly the glob behavior we want from the
        # put command.
        local_path = self._local_path.strip('*').strip(os.path.sep)
        return self._runner.put(os.path.join(local_path, '*'), release_path)


class LocalArtifactInstallation(ProjectBaseMixin):
    """Install a single local file into a remote release directory.

    This can be useful for installing applications that are typically bundled
    as a single file, e.g. Go binaries, Java JAR or WAR files, etc.. The artifact
    can optionally be renamed as part of the installation process.

    If the remote release directory does not already exist, it will be
    created during the install process.

    See :doc:`design` for more information about the expected directory
    structure for deployments.

    .. versionadded:: 1.1.0
    """

    def __init__(self, base, local_file, remote_name=None, runner=None):
        """Set the project base directory on the remote server, local artifact (a
        single file) that should be installed remotely, and optional file name
        to rename the artifact to on the remote server.

        :param str base: Absolute path to the root of the code deploy on
            the remote server
        :param str local_file: Relative or absolute path to the local artifact
            to be installed on the remote server.
        :param str remote_name: Optional file name for the artifact after it has
            been installed on the remote server. For example, if the artifact should
            always be called 'application.jar' on the remote server but might
            be named differently ('application-1.2.3.jar') locally, you would
            specify ``remote_name='application.jar'`` for this parameter.
        :param FabRunner runner: Optional runner to use for executing
            remote and local commands to perform the installation.
        :raises ValueError: If the base directory or local file isn't
            specified.
        """
        super(LocalArtifactInstallation, self).__init__(base)

        if not local_file:
            raise ValueError("You must specify a local file path")

        self._local_file = local_file
        self._remote_name = remote_name
        self._runner = runner if runner is not None else FabRunner()

    def install(self, release_id):
        """Install the local artifact into the remote release directory, optionally
        with a different name than the artifact had locally.

        If the directory for the given release ID does not exist on the remote
        system, it will be created. The directory will be created according to
        the standard Tunic directory structure (see :doc:`design`).

        :param str release_id: Timestamp-based identifier for this
            deployment.
        :return: The results of the ``put`` command using Fabric. This return
            value is an iterable of the paths of all files uploaded on the remote
            server.
        """
        release_path = os.path.join(self._releases, release_id)
        if not self._runner.exists(release_path):
            self._runner.run("mkdir -p '{0}'".format(release_path))

        # The artifact can optionally be renamed when being uploaded to
        # remote server. Useful for when we need a consistent name for
        # each deploy on the remote server but the local artifact includes
        # version numbers or something.
        if self._remote_name is not None:
            destination = os.path.join(release_path, self._remote_name)
        else:
            destination = release_path

        return self._runner.put(self._local_file, destination)


# pylint: disable=too-few-public-methods
class LocalArtifactTransfer(object):
    """Transfer a local artifact or directory of artifacts to a remote
    server when entering a context manager and clean the transferred files
    up on the remote server after leaving the block.

    The value yielded when entering the context manager will be the
    full path to the transferred file or directory on the remote
    server. The value yielded will be made up of ``remote_path``
    combined with the right most component of ``local_path``.

    For example, if ``/tmp/myapp`` is a local directory that contains
    several files, the example below will have the following effect.

    >>> transfer = LocalArtifactTransfer('/tmp/myapp', '/tmp/artifacts')
    >>> with transfer as remote_dest:
    ...     pass

    The directory ``myapp`` and its contents would be copied to ``/tmp/artifacts/myapp``
    on the remote machine within the scope of the context manager and
    the value of ``remote_dest`` would be ``/tmp/artifacts/myapp``. After
    the context manager exits ``/tmp/artifacts/myapp`` on the remote machine
    will be removed.

    If ``/tmp/myartifact.zip`` is a single local file, the example below
    will have the following effect.

    >>> transfer = LocalArtifactTransfer('/tmp/myartifact.zip', '/tmp/artifacts')
    >>> with transfer as remote_dest:
    ...     pass

    The file ``myartifact.zip`` would be copied to ``/tmp/artifacts/myartifact.zip``
    on the remote machine within the scope of the context manager and the
    value of ``remote_dest`` would be ``/tmp/artifacts/myartifact.zip``. After
    the context manager exits ``/tmp/artifacts/myartifact.zip`` on the remote
    machine will be removed.

    The destination of the artifacts must be a directory that is writable
    by the user running the deploy or that the user has permission to create.

    The path yielded by the context  will be removed when the context
    manager exits. The local artifacts are not modified or removed on
    exit.

    .. versionadded:: 0.4.0
    """

    def __init__(self, local_path, remote_path, runner=None):
        """Set the local directory that contains the artifacts and
        the remote directory that they should be transferred to.

        Both the local and remote paths should not contain trailing
        slashes. Any trailing slashes will be removed.

        :param str local_path: Directory path on the local machine that
            contains the build artifacts to be transferred (without a
            trailing slash) or the path on the local machine of a single
            file.
        :param str remote_path: Directory on the remote machine that
            the build artifacts should be transferred to (without a
            tailing slash).
        :param FabRunner runner: Optional runner to use for executing
            commands to transfer artifacts.

        .. versionchanged:: 0.5.0
            Trailing slashes are now removed from ``local_path`` and
            ``remote_path``.
        """
        self._local_path = local_path.rstrip(os.path.sep)
        self._remote_path = remote_path.rstrip(os.path.sep)
        self._remote_dest = os.path.join(
            self._remote_path, os.path.basename(self._local_path))
        self._runner = runner if runner is not None else FabRunner()

    def __enter__(self):
        """Transfer the local artifacts to the appropriate place on
        the remote server (ensuring the path exists first) and return
        the remote destination path.

        The remote destination path is the remote path joined with the
        right-most component of the local path.

        :return: The path artifacts were transferred to on the remote
            server
        :rtype: str
        """
        self._runner.run("mkdir -p '{0}'".format(self._remote_path))
        self._runner.put(self._local_path, self._remote_path)
        return self._remote_dest

    def __exit__(self, exc_type, exc_val, exc_tb):
        """Remove the directory containing the build artifacts on the
        remote server.
        """
        self._runner.run("rm -rf '{0}'".format(self._remote_dest))
        return False




# -*- coding: utf-8 -*-
#
# Tunic
#
# Copyright 2014-2015 TSH Labs <projects@tshlabs.org>
#
# Available under the MIT license. See LICENSE for details.
#

"""
tunic.core
~~~~~~~~~~

Core Tunic functionality.
"""

import uuid
from datetime import datetime

import os.path


try:
    # Fabric isn't available when our modules are imported during
    # documentation build on readthedocs.org so ignore the error
    # here if we are, in fact, on rtd.
    from fabric.api import (
        put,
        run,
        sudo)

    from fabric.contrib.files import exists
except ImportError as e:
    if os.getenv('READTHEDOCS', None) != 'True':
        raise
    put = None
    run = None
    sudo = None
    exists = None

PERMS_FILE_DEFAULT = 'u+rw,g+rw,o+r'
PERMS_DIR_DEFAULT = 'u+rwx,g+rws,o+rx'
RELEASE_DATE_FMT = '%Y%m%d%H%M%S'

# pylint: disable=invalid-name
_strip_all = lambda parts: [part.strip() for part in parts]


def split_by_line(content):
    """Split the given content into a list of items by newline.

    Both \r\n and \n are supported. This is done since it seems
    that TTY devices on POSIX systems use \r\n for newlines in
    some instances.

    If the given content is an empty string or a string of only
    whitespace, an empty list will be returned. If the given
    content does not contain any newlines, it will be returned
    as the only element in a single item list.

    Leading and trailing whitespace is remove from all elements
    returned.

    :param str content: Content to split by newlines
    :return: List of items that were separated by newlines.
    :rtype: list
    """
    # Make sure we don't end up splitting a string with
    # just a single trailing \n or \r\n into multiple parts.
    stripped = content.strip()

    if not stripped:
        return []
    if '\r\n' in stripped:
        return _strip_all(stripped.split('\r\n'))
    if '\n' in stripped:
        return _strip_all(stripped.split('\n'))
    return _strip_all([stripped])


def get_current_path(base):
    """Construct the path to the 'current release' symlink based on
    the given project base path.

    Note that this function does not ensure that the 'current' symlink
    exists or points to a valid release, it only returns the full path
    that it should be at based on the given project base directory.

    See :doc:`design` for more information about the expected directory
    structure for deployments.

    .. versionchanged:: 1.0.2
        :class:`ValueError` is now raised for empty ``base`` values.

    :param str base: Project base directory (absolute path)
    :raises ValueError: If the base directory isn't specified
    :return: Path to the 'current' symlink
    :rtype: str
    """
    if not base:
        raise ValueError("You must specify a project base directory")
    return os.path.join(base, 'current')


def get_releases_path(base):
    """Construct the path to the directory that contains all releases
    based on the given project base path.

    Note that this function does not ensure that the releases directory
    exists, it only returns the full path that it should be at based on
    the given project base directory.

    See :doc:`design` for more information about the expected directory
    structure for deployments.

    .. versionchanged:: 1.0.2
        :class:`ValueError` is now raised for empty ``base`` values.

    :param str base: Project base directory (absolute path)
    :raises ValueError: If the base directory isn't specified
    :return: Path to the releases directory
    :rtype: str
    """
    if not base:
        raise ValueError("You must specify a project base directory")
    return os.path.join(base, 'releases')


def get_release_id(version=None):
    """Get a unique, time-based identifier for a deployment
    that optionally, also includes some sort of version number
    or release.

    If a version is supplied, the release ID will be of the form
    '$timestamp-$version'. For example,  ``20140214231159-1.4.1``.

    If the version is not supplied the release ID will be of the
    form '$timestamp'. For example, ``20140214231159``.

    The timestamp component of this release ID will be generated
    using the current time in UTC.

    :param str version: Version to include in the release ID
    :return: Unique name for this particular deployment
    :rtype: str
    """
    ts = datetime.utcnow().strftime(RELEASE_DATE_FMT)

    if version is None:
        return ts
    return '{0}-{1}'.format(ts, version)


class FabRunner(object):
    """Wrapper for executing Fabric commands that allows us
    to globally disable use of shell wrappers and allow for
    easy testing.

    Note that if ``shell=True`` is not explicitly passed to
    methods in this class, use of a shell will be disabled.
    This is largely done in order to make sure that commands
    executed using ``sudo`` match any previously set grants
    (where as wrapping them with /bin/bash -c 'cmd' would
    break grants).
    """

    @staticmethod
    def run(*args, **kwargs):
        """Execute the Fabric :func:`run` function with the given args."""
        if 'shell' not in kwargs:
            kwargs['shell'] = False
        return run(*args, **kwargs)

    @staticmethod
    def sudo(*args, **kwargs):
        """Execute the Fabric :func:`sudo` function with the given args."""
        if 'shell' not in kwargs:
            kwargs['shell'] = False
        return sudo(*args, **kwargs)

    @staticmethod
    def exists(*args, **kwargs):
        """Execute the Fabric :func:`fabric.contrib.files.exists` function
        with the given args.
        """
        return exists(*args, **kwargs)

    @staticmethod
    def put(*args, **kwargs):
        """Execute the Fabric :func:`put` function with the given args."""
        return put(*args, **kwargs)


# pylint: disable=too-few-public-methods
class ProjectBaseMixin(object):
    """Base for setting project directories from a given
    project root directory.

    :ivar str _base: Project root directory
    :ivar str _current: "current release" symlink
    :ivar str _release: Directory of all releases
    """

    def __init__(self, base):
        """Set the project directories based on a given root.

        :param str base: Project root directories.
        :raises ValueError: If ``base`` is None or empty
        """
        if not base:
            raise ValueError("You must specify a project base directory")

        self._base = base
        self._current = get_current_path(base)
        self._releases = get_releases_path(base)


class ReleaseManager(ProjectBaseMixin):
    """Functionality for manipulation of multiple releases of a project
    deployed on a remote server.

    Note that functionality for managing releases relies on them being
    named with a timestamp based prefix that allows them to be naturally
    sorted -- such as with the :func:`get_release_id` function.

    See :doc:`design` for more information about the expected directory
    structure for deployments.
    """

    def __init__(self, base, runner=None):
        """Set the base path to the project that we will be managing
        releases of and an optional :class:`FabRunner` implementation
        to use for running commands.

        :param str base: Absolute path to the root of the code deploy
        :param FabRunner runner: Optional runner to use for executing
            remote commands to manage releases.
        :raises ValueError: If the base directory isn't specified

        .. versionchanged:: 0.3.0
            :class:`ValueError` is now raised for empty ``base`` values.
        """
        super(ReleaseManager, self).__init__(base)
        self._runner = runner if runner is not None else FabRunner()

    def get_current_release(self):
        """Get the release ID of the "current" deployment, None if
        there is no current deployment.

        :return: Get the current release ID
        :rtype: str
        """
        current = self._runner.run("readlink '{0}'".format(self._current))
        if current.failed:
            return None
        return os.path.basename(current.strip())

    def get_releases(self):
        """Get a list of all previous deployments, newest first.

        :return: Get an ordered list of all previous deployments
        :rtype: list
        """
        return split_by_line(
            self._runner.run("ls -1r '{0}'".format(self._releases)))

    def get_previous_release(self):
        """Get the release ID of the deployment immediately
        before the "current" deployment, ``None`` if no previous
        release could be determined.

        :return: The release ID of the release previous to the
            "current" release.
        :rtype: str
        """
        releases = self.get_releases()
        if not releases:
            return None

        current = self.get_current_release()
        if not current:
            return None

        try:
            current_idx = releases.index(current)
        except ValueError:
            return None

        try:
            return releases[current_idx + 1]
        except IndexError:
            return None

    def set_current_release(self, release_id):
        """Change the 'current' symlink to point to the given
        release ID.

        The 'current' symlink will be updated in a way that ensures
        the switch is done atomically.

        :param str release_id: Release ID to mark as the current
            release
        """
        self._set_current_release(release_id, str(uuid.uuid4()))

    # pylint: disable=missing-docstring
    def _set_current_release(self, release_id, rand):
        # Set the current release with a specific random file
        # name. Useful for unit testing when we need to check
        # for the exact arguments passed to the runner mock
        tmp_path = os.path.join(self._base, rand)
        target = os.path.join(self._releases, release_id)

        # First create a link with a random name to point to the
        # newly created release, then rename it to 'current' such
        # that the symlink is updated atomically [1].
        # [1] - http://rcrowley.org/2010/01/06/things-unix-can-do-atomically
        self._runner.run("ln -s '{0}' '{1}'".format(target, tmp_path))
        self._runner.run("mv -T '{0}' '{1}'".format(tmp_path, self._current))

    def cleanup(self, keep=5):
        """Remove all but the ``keep`` most recent releases.

        If any of the candidates for deletion are pointed to by the
        'current' symlink, they will not be deleted.

        :param int keep: Number of old releases to keep around
        """
        releases = self.get_releases()
        current_version = self.get_current_release()
        to_delete = [version for version in releases[keep:] if version != current_version]

        for release in to_delete:
            self._runner.run("rm -rf '{0}'".format(os.path.join(self._releases, release)))


class ProjectSetup(ProjectBaseMixin):
    """Functionality for performing the initial creation of project
    directories and making sure their permissions are reasonable on
    a remote server.

    Note that by default methods in this class rely on being able to
    execute commands with the Fabric ``sudo`` function. This can be
    disabled by passing the ``use_sudo=False`` flag to methods that
    accept it.

    See :doc:`design` for more information about the expected directory
    structure for deployments.
    """

    def __init__(self, base, runner=None):
        """Set the base path to the project that will be setup and an
        optional :class:`FabRunner` implementation to use for running
        commands.

        :param str base: Absolute path to the root of the code deploy
        :param FabRunner runner: Optional runner to use for executing
            remote commands to set up the deploy.
        :raises ValueError: If the base directory isn't specified

        .. versionchanged:: 0.3.0
            :class:`ValueError` is now raised for empty ``base`` values.
        """
        super(ProjectSetup, self).__init__(base)
        self._runner = runner if runner is not None else FabRunner()

    def setup_directories(self, use_sudo=True):
        """Create the minimal required directories for deploying multiple
        releases of a project.

        By default, creation of directories is done with the Fabric
        ``sudo`` function but can optionally use the ``run`` function.

        :param bool use_sudo: If ``True``, use ``sudo()`` to create required
            directories. If ``False`` try to create directories using the
            ``run()`` command.
        """
        runner = self._runner.sudo if use_sudo else self._runner.run
        runner("mkdir -p '{0}'".format(self._releases))

    def set_permissions(
            self, owner, file_perms=PERMS_FILE_DEFAULT,
            dir_perms=PERMS_DIR_DEFAULT, use_sudo=True):
        """Set the owner and permissions of the code deploy.

        The owner will be set recursively for the entire code deploy.

        The directory permissions will be set on only the base of the
        code deploy and the releases directory. The file permissions
        will be set recursively for the entire code deploy.

        If not specified default values will be used for file or directory
        permissions.

        By default the Fabric ``sudo`` function will be used for changing
        the owner and permissions of the code deploy. Optionally, you can
        pass the ``use_sudo=False`` argument to skip trying to change the
        owner of the code deploy and to use the ``run`` function to change
        permissions.

        :param str owner: User and group in the form 'owner:group' to
            set for the code deploy.
        :param str file_perms: Permissions to set for all files in the
            code deploy in the form 'u+perms,g+perms,o+perms'. Default
            is ``u+rw,g+rw,o+r``.
        :param str dir_perms: Permissions to set for the base and releases
            directories in the form 'u+perms,g+perms,o+perms'. Default
            is ``u+rwx,g+rws,o+rx``.
        :param bool use_sudo: If ``True``, use ``sudo()`` to change ownership
            and permissions of the code deploy. If ``False`` try to change
            permissions using the ``run()`` command, do not change ownership.

        .. versionchanged:: 0.2.0
            ``use_sudo=False`` will no longer attempt to change ownership of
            the code deploy since this will just be a no-op or fail.

        """
        runner = self._runner.sudo if use_sudo else self._runner.run

        if use_sudo:
            runner("chown -R '{0}' '{1}'".format(owner, self._base))

        for path in (self._base, self._releases):
            runner("chmod '{0}' '{1}'".format(dir_perms, path))

        runner("chmod -R '{0}' '{1}'".format(file_perms, self._base))



# -*- coding: utf-8 -*-
#
# Tunic
#
# Copyright 2014-2015 TSH Labs <projects@tshlabs.org>
#
# Available under the MIT license. See LICENSE for details.
#

"""
tunic.api
~~~~~~~~~

Publicly exposed Tunic methods.
"""

from .core import (
    get_current_path,
    get_releases_path,
    get_release_id,
    ReleaseManager,
    ProjectSetup)

from .install import (
    LocalArtifactTransfer,
    LocalArtifactInstallation,
    StaticFileInstallation,
    VirtualEnvInstallation)

__all__ = [
    'get_current_path',
    'get_releases_path',
    'get_release_id',
    'ReleaseManager',
    'ProjectSetup',
    'LocalArtifactInstallation',
    'StaticFileInstallation',
    'VirtualEnvInstallation',
    'LocalArtifactTransfer'
]



# -*- coding: utf-8 -*-
#

from __future__ import print_function
import codecs

from setuptools import setup, find_packages

import tunic


DESCRIPTION = 'Deployment related Fabric utilities'
AUTHOR = 'TSH Labs'
EMAIL = 'projects@tshlabs.org'
URL = 'http://www.tshlabs.org/'
LICENSE = 'MIT'
CLASSIFIERS = [
    "Development Status :: 5 - Production/Stable",
    "License :: OSI Approved :: MIT License",
    "Operating System :: POSIX",
    "Programming Language :: Python",
    "Programming Language :: Python :: 2.6",
    "Programming Language :: Python :: 2.7",
    "Topic :: System :: Installation/Setup"
]


def get_contents(filename):
    with codecs.open(filename, 'rb', encoding='utf-8') as handle:
        return handle.read()


REQUIRES = [
    'fabric'
]

README = get_contents('README.rst')

setup(
    name='tunic',
    version=tunic.__version__,
    author=AUTHOR,
    description=DESCRIPTION,
    long_description=README,
    author_email=EMAIL,
    classifiers=CLASSIFIERS,
    license=LICENSE,
    url=URL,
    zip_safe=True,
    install_requires=REQUIRES,
    packages=find_packages())



# -*- coding: utf-8 -*-
#
# Tunic
#
# Copyright 2014-2015 TSH Labs <projects@tshlabs.org>
#
# Available under the MIT license. See LICENSE for details.
#

"""
tunic
~~~~~

Deployment related Fabric utilities.
"""

__version__ = '1.1.0'
